--------------------------------------------------------------------------------
-- Global code
--------------------------------------------------------------------------------
require('mathlib')

local state = 'stalk'
local van = nil
local dropoff = nil
local player = nil
local vantage = nil
local dealer = nil

local function startGlobal()
	state = 'stalk'
	van     = scenetree.findObject('drug_van')
	dropoff = scenetree.findObject('drug_dropoff')
	player  = scenetree.findObject('scenario_player0')
	vantage = scenetree.findObject('drug_vantage')
	dealer  = scenetree.findObject('drug_dealer')
end

local function fail(reason)
    scenarios.finish({failed = reason})
end

local function success(reason)     
    scenarios.finish({msg = reason})
end

--------------------------------------------------------------------------------
-- Stalking code
--------------------------------------------------------------------------------

-- The minimum distance to stay away from the van (meter)
local stalkMinDistance = 40

-- The furthest you're allowed to go from the van (meter)
local stalkMaxDistance = 150

-- How long you're allowed to stay out of the allowed distance (Seconds)
local stalkMaxTimeout = 5

-- How close the van has to get to the dropoff location
local stalkDropoffDistance = 10

-- Variables
local shownStalkMessage = false
local shownDistanceMessage = false
local isTargetSet = false
local isModeSet = false
local stalkTimeout = 0
local stalkFailReason = ''

local function startStalk()
	shownStalkMessage = false
	shownDistanceMessage = false
	isTargetSet = false
	isModeSet = false
	stalkTimeout = 0
	stalkFailReason = ''
end

local function updateStalk(raceTickTime)

	if not van or not dropoff or not player then
		return
	end
		
	if not shownStalkMessage then
		guihooks.trigger('ScenarioFlashMessage', {{"Follow the van", 2}} )
		shownStalkMessage = true
	end
	
	if not isModeSet then
	
		-- Ensure the van doesn't drive like a *complete* madman
		van:queueLuaCommand('ai.setAggression(0.8)')
		van:queueLuaCommand('ai.setState({ mode = "manual" })')
		isModeSet = true
	
	elseif not isTargetSet then
	
		van:queueLuaCommand('ai.setTarget("drug_dropoff")')
		isTargetSet = true
	end
	
	local vanPosition = van:getPosition()
	local dropoffPosition = dropoff:getPosition()
	local playerPosition = player:getPosition()
	local playerDistance = math.abs((playerPosition - vanPosition):len())
	
	if playerDistance < stalkMinDistance then
		
		if not shownDistanceMessage then
			guihooks.trigger('ScenarioFlashMessage', {{"Don't let him notice you!", 2}} )
			shownDistanceMessage = true
			stalkFailReason = 'The van driver spotted you! Stay further away next time.'
		end
		
		stalkTimeout = stalkTimeout + raceTickTime
		
	elseif playerDistance > stalkMaxDistance then
		
		if not shownDistanceMessage then
			guihooks.trigger('ScenarioFlashMessage', {{"Don't lose the van!", 2}} )
			shownDistanceMessage = true
			stalkFailReason = 'You lost the van! Make sure you stay close enough to see him.'
		end
		
		stalkTimeout = stalkTimeout + raceTickTime
	
	else
	
		stalkTimeout = 0
		shownDistanceMessage = false
		
	end
	
	if stalkTimeout > stalkMaxTimeout then
		fail(stalkFailReason)
	end
	
	local dropoffDistance = math.abs((dropoffPosition - vanPosition):len())
	
	if dropoffDistance < stalkDropoffDistance then
		state = 'vantage'
	end
end

--------------------------------------------------------------------------------
-- Vantage point code
--------------------------------------------------------------------------------

-- How close the player has to get to the vantage point
local vantageMaxDistance = 2.5

-- How long before the dealer flees
local vantageMaxTimeout = 10

-- How close the player is allowed to get to the dealer
local vantageMinDealerDistance = 35

-- Variables
local shownGotoMessage = false
local shownWaitMessage = false
local setVantageVisible = false
local vantageTimeout = 0

local function startVantage()
	shownGotoMessage = false
	shownWaitMessage = false
	setVantageVisible = false
	vantageTimeout = 0
end

local function updateVantage(raceTickTime)
	if not vantage or not player or not dealer then
		return
	end
	
	if not shownGotoMessage then
		guihooks.trigger('ScenarioFlashMessage',
			{
				{ "The van has arrived", 3 },
				{ "Wait at the marker", 3 }
			})
		shownGotoMessage = true
	end
	
	local vantagePosition = vantage:getPosition()
	local playerPosition = player:getPosition()
	local dealerPosition = dealer:getPosition()
	
	local dealerDistance = math.abs((playerPosition - dealerPosition):len())
	
	-- We want to ignore height differences
	vantagePosition.z = 0
	playerPosition.z = 0
	
	local vantageDistance = math.abs((playerPosition - vantagePosition):len())
		
	if dealerDistance < vantageMinDealerDistance then
		fail("Don't drive into the deal! What were you thinking?")
	end
	
	if vantageDistance < vantageMaxDistance then
		
		if not shownWaitMessage then
			guihooks.trigger('ScenarioFlashMessage', {{ "Watch the deal go down", 3 }})
			shownWaitMessage = true
			vantage.isRenderEnabled = false
			setVantageVisible = false
		end
		
		vantageTimeout = vantageTimeout + raceTickTime
	
	else
	
		stalkTimeout = 0
		shownWaitMessage = false
		
		if not setVantageVisible then
			vantage.isRenderEnabled = true
			setVantageVisible = true
		end
		
	end
	
	if vantageTimeout > vantageMaxTimeout then
	
		guihooks.trigger('ScenarioFlashMessage',
			{
				{ "You've been spotted!", 2 },
				{ "Don't let him get away!", 2 }
			})
		dealer:queueLuaCommand('ai.setState({ mode = "flee" })')
		state = 'chase'
		
	end
end

--------------------------------------------------------------------------------
-- Chase code
--------------------------------------------------------------------------------

-- The minimum speed the dealer needs to have
local chaseMinSpeed = 1

-- How long before the dealer is timed out
local chaseMaxTimeout = 5

-- How far away the dealer has to get for the player to lose
local chaseMaxDistance = 200

-- Variables
local chaseTimeout = 0

local function startChase()
	shownGotoMessage = false
	shownWaitMessage = false
	vantageTimeout = 0
end

local function updateChase(raceTickTime)
	if not dealer or not player then
		return
	end
	
	local dealerPosition = dealer:getPosition()
	local playerPosition = player:getPosition()
	local distance = math.abs((playerPosition - dealerPosition):len())
	
	if distance > chaseMaxDistance then
		fail('The dealer got away!')
	end
	
	local dealerData = map.objects.drug_dealer
	
	if dealerData then
		local dealerSpeed = map.objects.drug_dealer.vel:length()
		
		if dealerSpeed < chaseMinSpeed then
			
			chaseTimeout = chaseTimeout + raceTickTime
			
			if chaseTimeout % 1 == 0 then
				local countdown = chaseMaxTimeout - math.modf(chaseTimeout)
				
				if countdown > 0 then
					guihooks.trigger('ScenarioFlashMessage', {{countdown .. '...', 0.5}} )
				end
			end
		
		else
		
			chaseTimout = 0
		
		end
		
		if chaseTimeout > chaseMaxTimeout then
			success('You took out the dealer! Well done!')
		end
	else
		dealer:queueLuaCommand('mapmgr.enableTracking("drug_dealer")')
	end
end

--------------------------------------------------------------------------------
-- Scenario code
--------------------------------------------------------------------------------
local M = {}

local function onScenarioChange(scenario)

	-- Create a node for the dropoff waypoint if it doesn't exist
	if map.map.nodes['drug_dropoff'] ~= nil then
		return
	end
	
	local wp = scenetree.findObject('drug_dropoff')
	local position = wp:getPosition()
	
	local bestNode = nil
	local bestDist = math.huge
	
	for nodeName, node in pairs(map.map.nodes) do
		local distance = math.abs((position - node.pos):len())
		
		if distance < bestDist then
			bestDist = distance
			bestNode = nodeName
		end
	end
	
	map.map.nodes['drug_dropoff'] =
	{
		pos = position,
		radius = wp:getScale():len() * 0.5,
		links = { [bestNode] = 1 },
		drivability = 1,
		segTags = {},
		manual = 1,
	}
end

local function onRaceStart()
	startGlobal()
	startStalk()
	startVantage()
	startChase()
end

local function onRaceTick(raceTickTime)
	if state == 'stalk' then
		updateStalk(raceTickTime)
	elseif state == 'vantage' then
		updateVantage(raceTickTime)
	elseif state == 'chase' then
		updateChase(raceTickTime)
	end
end

M.onScenarioChange = onScenarioChange
M.onRaceStart = onRaceStart
M.onRaceTick = onRaceTick

return M